"use strict";
exports.__esModule = true;
var svgoConfig = require('../../svgo.config.json');
var utils_1 = require("./utils");
var config_1 = require("../config");
var env_1 = require("../env");
// Constants
var ESLINT_RC = './node_modules/jk-standard/.eslintrc.json';
var TSLINT_RC = './node_modules/jk-standard/tslint.json';
var EXCLUDE_REG = /node_modules/;
var SVG_SYMBOL_REGS = [
    /jk-icons\/lib\/svgs/,
    /svg-symbols/
];
exports.BABEL_OPTIONS = {
    presets: [
        ['es2015', {
                loose: true,
                modules: false
            }],
        'stage-0',
        'react'
    ],
    plugins: [
        'transform-runtime',
        'transform-decorators-legacy',
        'syntax-dynamic-import',
        ['import', [{
                    libraryName: 'antd-mobile'
                }]]
    ],
    cacheDirectory: true
};
var IMAGEMIN_OPTIONS = {
    plugins: [
        {
            use: 'imagemin-pngquant',
            options: {
                quality: '60-80'
            }
        }
    ]
};
exports.cssLoaderOptions = function () { return ({
    extract: env_1.$isProd,
    sourceMap: !env_1.$isProd,
    cssModules: config_1.CONFIG.CSS_MODULES
}); };
// Loaders
var eslintLoader = {
    enforce: 'pre',
    test: /\.(jsx?|vue)$/,
    loader: 'eslint-loader',
    exclude: EXCLUDE_REG,
    options: {
        configFile: ESLINT_RC
    }
};
var tslintLoader = {
    enforce: 'pre',
    test: /\.tsx?$/,
    loader: 'tslint-loader',
    exclude: EXCLUDE_REG,
    options: {
        configFile: TSLINT_RC
    }
};
var jsonLoader = {
    test: /\.json$/,
    loader: 'json-loader'
};
var fontLoader = {
    test: /\.(woff2?|eot|ttf|otf)(\?.*)?$/,
    loader: 'url-loader',
    options: {
        limit: 10000,
        name: utils_1.assetsPath('fonts/[name].[hash:7].[ext]')
    }
};
var imageLoader = {
    test: /\.(png|jpe?g|svg|gif)(\?.*)?$/,
    loader: 'file-loader',
    exclude: SVG_SYMBOL_REGS,
    options: {
        limit: 10000,
        name: utils_1.assetsPath('images/[name].[hash:7].[ext]')
    }
};
var pugLoader = {
    test: /\.pug$/,
    loader: 'pug-loader'
};
var svgSymbolLoader = {
    test: /\.svg$/i,
    loader: 'svg-sprite-loader',
    include: SVG_SYMBOL_REGS,
    options: {
        symbolId: 'symbol-[name]'
    }
};
var imageminLoader = {
    test: /\.png$/,
    enforce: 'pre',
    loader: 'imagemin-loader',
    exclude: EXCLUDE_REG,
    options: IMAGEMIN_OPTIONS
};
var svgoLoader = {
    test: /\.svg$/,
    enforce: 'pre',
    loader: 'svgo-loader',
    exclude: /node_modules/,
    options: svgoConfig
};
var jsLoader = {
    test: /\.jsx?$/,
    loader: 'babel-loader',
    exclude: EXCLUDE_REG,
    options: exports.BABEL_OPTIONS
};
var tsLoader = {
    test: /\.tsx?$/,
    loader: 'ts-loader',
    exclude: EXCLUDE_REG
};
var vueLoader = {
    test: /\.vue?$/,
    loader: 'vue-loader'
};
var cssLoaders = function () { return utils_1.styleRules(exports.cssLoaderOptions()); };
// Loader lists
var preLoaders = [
    eslintLoader,
    tslintLoader
];
var templateLoaders = [
    pugLoader
];
var staticLoaders = [
    jsonLoader,
    fontLoader,
    imageLoader,
    svgSymbolLoader
];
var staticOptimizeLoaders = [
    imageminLoader,
    svgoLoader
];
var compileLoaders = function () { return ([
    jsLoader,
    tsLoader,
    vueLoader
].concat(cssLoaders())); };
// @Env: Prod
if (env_1.$isProd) {
    staticLoaders.push.apply(staticLoaders, staticOptimizeLoaders);
}
// @Export
exports.WEBPACK_LOADERS = function () { return preLoaders.concat(templateLoaders, staticLoaders, compileLoaders()); };
