"use strict";
exports.__esModule = true;
var ExtractTextPlugin = require('extract-text-webpack-plugin');
var OptimizeCssAssetsPlugin = require('optimize-css-assets-webpack-plugin');
var HtmlWebpackPlugin = require('html-webpack-plugin');
var VueLoaderOptionsPlugin = require('vue-loader-options-plugin');
var fs = require("fs");
var path = require("path");
var webpack_1 = require("webpack");
var loaders_1 = require("./loaders");
var utils_1 = require("./utils");
var config_1 = require("../config");
var env_1 = require("../env");
// Constants
var POSTCSS_CONFIG_NAME = 'postcss.config.js';
var POSTCSS_RC = fs.existsSync(env_1.PROJECT_ROOT + "/" + POSTCSS_CONFIG_NAME)
    ? env_1.PROJECT_ROOT + "/" + POSTCSS_CONFIG_NAME
    : path.resolve(__dirname, "../../" + POSTCSS_CONFIG_NAME);
// Loader Externel Options
var postCssOptions = function () {
    var options = [
        require('autoprefixer')({
            browsers: [
                'last 2 versions',
                'ie 9-11',
                'ios >= 8',
                'android >= 4.0'
            ]
        })
    ];
    if (config_1.CONFIG.PX_TO_REM) {
        options.push(require('postcss-pxtorem')({
            rootValue: 100
        }));
    }
    return options;
};
var vueOptions = {
    loaders: utils_1.cssLoaders(loaders_1.cssLoaderOptions)
};
// Plugins
var definePlugin = new webpack_1.DefinePlugin({
    'process.env': {
        NODE_ENV: JSON.stringify(env_1.ENV)
    },
    // Global variables
    __DEBUG__: !env_1.$isProd,
    __ENV__: JSON.stringify(env_1.ENV)
});
var loaderOptionsPlugin = function () { return new webpack_1.LoaderOptionsPlugin({
    options: {
        postcss: postCssOptions(),
        vue: vueOptions,
        context: __dirname
    }
}); };
var vueLoaderOptionsPlugin = new VueLoaderOptionsPlugin({
    babel: loaders_1.BABEL_OPTIONS
});
var defualtIndexHtmlPlugin = new HtmlWebpackPlugin();
var noEmirOnErrorsPlugin = new webpack_1.NoEmitOnErrorsPlugin();
var nameModulesPlugin = new webpack_1.NamedModulesPlugin();
var hotModuleReplacePlugin = new webpack_1.HotModuleReplacementPlugin();
var extractTextPlugin = new ExtractTextPlugin(utils_1.assetsPath('styles/[name].[contenthash].css'));
var compressCssPlugin = new OptimizeCssAssetsPlugin({
    cssProcessorOptions: {
        safe: true
    }
});
var compressJsPlugin = new webpack_1.optimize.UglifyJsPlugin({
    comments: false,
    compress: {
        dead_code: true,
        warnings: false
    }
});
var occurrenceOrderPlugin = new webpack_1.optimize.OccurrenceOrderPlugin(true);
// List
var devPlugins = [];
var buildPlugins = [];
if (!env_1.$isProd) {
    devPlugins.push(noEmirOnErrorsPlugin, nameModulesPlugin, hotModuleReplacePlugin);
}
if (env_1.$isProd) {
    buildPlugins.push(extractTextPlugin, compressCssPlugin, compressJsPlugin, occurrenceOrderPlugin);
}
exports.WEBPACK_PLUGINS = function () { return [
    definePlugin,
    loaderOptionsPlugin(),
    vueLoaderOptionsPlugin,
    defualtIndexHtmlPlugin
].concat(devPlugins, buildPlugins); };
