const loaderOptionsMerge  = require('webpack-loader-options-merge')

import { Configuration } from 'webpack'
import { WEBPACK_ENTRY } from './entry'
import { WEBPACK_OUTPUT } from './output'
import { WEBPACK_RESOLVE } from './resolve'
import { WEBPACK_DEV_TOOL, WEBPACK_DEV_SERVER } from './dev'
import { WEBPACK_LOADERS } from './loaders'
import { WEBPACK_PLUGINS } from './plugins'
import { ConfigSchema, updateConfig } from '../config'

export * from './entry'
export * from './output'
export * from './resolve'
export * from './dev'
export * from './loaders'
export * from './plugins'

export const webpackMerge = require('webpack-merge')

export const mergeWebpackConfig = (...args): Configuration => {
  const mergedConfig = webpackMerge(...args)
  return loaderOptionsMerge(mergedConfig)
}

export const createWebpackConfig = (bsConfig?: ConfigSchema): Configuration => {
  updateConfig(bsConfig)

  return {
    cache: true,

    // Entry
    entry: WEBPACK_ENTRY(),

    // Output
    output: WEBPACK_OUTPUT(),

    // Resolve
    resolve: WEBPACK_RESOLVE(),

    // Dev
    devtool: WEBPACK_DEV_TOOL(),
    devServer: WEBPACK_DEV_SERVER(),

    // Loaders
    module: {
      rules: WEBPACK_LOADERS()
    },

    // Plugins
    plugins: WEBPACK_PLUGINS()
  }
}
