const ExtractTextPlugin = require('extract-text-webpack-plugin')

import { CONFIG } from '../config'

export const assetsPath = (path) => {
  return `${CONFIG.ASSETS_DIR}/${path}`
}

export interface CSSLoaderOptions {
  extract?: boolean
  sourceMap?: boolean
  cssModules?: boolean
}

export const cssLoaders = (options?: CSSLoaderOptions) => {
  options = options || {}

  // generate loader string to be used with extract text plugin
  const generateLoaders = (loaders) => {
    const sourceLoader = loaders.map((loader) => {
      let extraParamChar = ''

      if (/\?/.test(loader)) {
        loader = loader.replace(/\?/, '-loader?')
        extraParamChar = '&'
      } else {
        loader = loader + '-loader'
        extraParamChar = '?'
      }

      if (options.sourceMap) {
        extraParamChar += 'sourceMap'
      }

      if (loader === 'css-loader' && options.cssModules) {
        extraParamChar += '&modules&localIdentName=[local]-[hash:base64:5]'
      }

      if (loader === 'stylus-loader') {
        if (options.cssModules) {
          loader += '&include css'
        }
      }

      return loader + extraParamChar
    }).join('!')

    if (options.extract) {
      return ExtractTextPlugin.extract({
        fallback: 'style-loader',
        use: sourceLoader
      })
    } else {
      return ['style-loader', sourceLoader].join('!')
    }
  }

  return {
    css: generateLoaders(['css', 'postcss']),
    scss: generateLoaders(['css', 'postcss', 'sass']),
    styl: generateLoaders(['css', 'postcss', 'stylus?resolve url'])
  }
}

// Generate rules for standalone style files
export const styleRules = (options?: CSSLoaderOptions) => {
  options = options || {}

  const output = []
  const NODE_MODULES = /node_modules/
  const loaders = cssLoaders(options)
  const action = options.extract ? 'use' : 'loader'
  const exclude = options.cssModules ? NODE_MODULES : /noparse-/

  if (options.cssModules) {
    const noModulesOptions = {
      ...options,
      cssModules: false
    }

    const noModulesLoaders = cssLoaders(noModulesOptions)

    for (const extension in noModulesLoaders) {
      const loader = noModulesLoaders[extension]
      const noModulesRule = {
        test: new RegExp('\\.' + extension + '$'),
        [`${action}`]: loader,
        include: NODE_MODULES
      }
      output.push(noModulesRule)
    }
  }

  for (const extension in loaders) {
    const loader = loaders[extension]
    const rule = {
      test: new RegExp('\\.' + extension + '$'),
      [`${action}`]: loader,
      exclude
    }
    output.push(rule)
  }

  return output
}
